weave = function() {

	var cdiv = 0;

	$(window).resize(function() {
		scalePreview();
	});

	function scalePreview() {
		$('#ipreview').height($(window).height());
	}

	function getClassName(tx) {
		return $.trim(tx.replace('{', ''));
	}

	function getTransform(t, m) {
		if(!t || t.length === 0)
			return false;
		var rt = [];
		for(e in t) {
			var f = t[e];
			var v = Math.round(parseFloat(f.v) * m * 10000) / 10000;
			var p = Math.round(Math.pow(parseFloat(f.v), m) * 10000) / 10000;
			if(f.t === 'x')
				rt.push('translateX(' + v + 'px)');
			else if(f.t === 'y')
				rt.push('translateY(' + v + 'px)');
			else if(f.t === 's')
				rt.push('scale(' + p + ')');
			else if(f.t === 'sx')
				rt.push('scaleX(' + p + ')');
			else if(f.t === 'sy')
				rt.push('scaleY(' + p + ')');
			else if(f.t === 'k')
				rt.push('skew(' + v + 'deg)');
			else if(f.t === 'kx')
				rt.push('skewX(' + v + 'deg)');
			else if(f.t === 'ky')
				rt.push('skewY(' + v + 'deg)');
			else if(f.t === 'r')
				rt.push('rotate(' + v + 'deg)');
		}
		if(rt.length === 0)
			return false;
		var cs = rt.join(' ');
		return '-moz-transform:' + cs +
						';-o-transform:' + cs +
						';-webkit-transform:' + cs + ';';
	}

	function getMultiFactor(tx) {
		var rp = parseInt(tx);
		return ((isNaN(rp) || rp < 1) ? false : rp);
	}

	function isValidLine(tx) {
		var sf = tx.substr(tx.length - 1, 1);
		return (sf ==='' || sf === '{' || sf === '}' || sf === ';');
	}

	function isClass(tx) {
		var pf = tx.substr(0, 1);
		var sf = tx.substr(tx.length - 1, 1);
		return ((pf === '.' || pf === '#') && sf === '{');
	}

	function isMultiObject(tx) {
		var rp = getMultiFactor(tx);
		if(!rp)
			return false;
		var dt = tx.indexOf('.');
		var sf = tx.substr(tx.length - 1, 1);
		return ((tx === rp + tx.substr(dt)) && (tx.indexOf(':') > -1 || sf === ';'));
	}

	function isObject(tx) {
		var pf = tx.substr(0, 1);
		var sf = tx.substr(tx.length - 1, 1);
		return (pf === '.' && (tx.indexOf(':') > -1 || sf === ';'));
	}

	function isProperty(tx) {
		return (tx.substr(0, 1) === '@');
	}

	function splitPropVal(tx) {
		tx = tx.replace(';', '');
		var sp = tx.indexOf(':');
		if(sp === -1)
			return [tx, ''];
		return [$.trim(tx.substr(0, sp)), $.trim(tx.substr(sp+1, tx.length))];
	}

	function splitClassVal(tx) {

		var rp = getMultiFactor(tx);
		if(rp)
			tx = tx.substr(tx.indexOf('.'));
		else
			rp = 1;

		var pv = splitPropVal(tx);
		var t = pv[1];
		while(t.indexOf('  ') > -1)
			t = t.replace(/  /g, ' ');
			if(t === '') {
				t = [];
			} else {
				var v = t.split(' ');
				if(v.length % 2 !== 0)
					return false;
				t = [];
				for(var i = 0; i < v.length; i+=2) {
					var vp = v[i];
					var vv = v[i+1];
					if($.inArray(vp, ['x', 'y', 's', 'sx', 'sy', 'k', 'kx', 'ky', 'r']) === -1)
						return false;
					t.push({t:vp, v:vv});
				}
			}
		return {'o': pv[0], 't': t, 'r': rp};
	}

	function pickClassNum(cl) {
		if(cl.length === 1)
			return 0;
		var pr = [];
		var pm = 0;
		var cm = 0;
		for(i in cl) {
			pm = (('@prob' in cl[i]) ? cl[i]['@prob'] : 0.5);
			cm += parseFloat(pm);
			pr[i] = cm;
		}
		var rn = Math.random() * cm;
		var sl = 0;
		for(i in pr) {
			sl = i;
			if(pr[i] > rn)
				break;
		}
		return sl;
	}

	function parseSource(src, def, err, css) {
		var arc = $.trim(src).split('\n');
		var cc = false;
		var ccn = 0;
		for(lc in arc) {
			var rx = $.trim(arc[lc]);
			var tx = rx.toLowerCase();
			if(!isValidLine(tx)) {
				err.push([lc, arc[lc],
									'Each line must end in one of these characters: { } ;']);
			} else if(isClass(tx)) {
				var cn = getClassName(tx);
				if(cn.substr(0, 1) === '#' && cn !== '#stage') {
					err.push([lc, arc[lc],
										'Only #stage can be defined with \'#\'. Use \'.\' for everything else.']);
				} else {
					cc = cn;
					if(!(cn in def))
						def[cn] = [];
					ccn = def[cn].length;
					if(cn === '#stage' && ccn > 0) {
						err.push([lc, arc[lc],
										'Only one #stage can be defined.']);
						ccn = 0;
					} else {
						var cl = (cn !== '#stage' ? $.trim(tx.replace('{', '')) + ccn + ' {' : tx);
						css.push(cl);
						def[cn][ccn] = {'code':[]};
					}
				}
			} else if(isObject(tx) || isMultiObject(tx)) {
				if(cc) {
					var cv = splitClassVal(tx);
					if(!cv)
						err.push([lc, arc[lc], 'Invalid transform parameters.']);
					else
						def[cn][ccn]['code'].push(cv);
				} else {
					err.push([lc, arc[lc], 'Object must be part of a class definition.']);
				}
			} else if(isProperty(tx)) {
				if(cc) {
					var pv = splitPropVal(tx);
					def[cn][ccn][pv[0]] = pv[1];
				} else {
					err.push([lc, arc[lc], 'Property must be part of a class definition.']);
				}
			} else {
				if(tx === '}') {
					css.push(rx);
					cc = false;
				} else if(cc) {
					css.push(rx);
				} else if(tx !== '') {
					err.push([lc, arc[lc], 'CSS must be part of a class definition.']);
				}
			}
		}

		if(cc)
			err.push(['', '', 'Missing ending }']);
	}

	function renderStage(def, err, htm, obj, mdep, mdiv) {
		if(!(obj in def))
			return err.push(['', obj, 'Missing class definition!']);
		var o = def[obj][0];
		if('@maxdepth' in o)
			mdep = parseInt(o['@maxdepth']);
		if('@maxhtml' in o)
			cdiv = parseInt(o['@maxhtml']);
		else
			cdiv = mdiv;
		if(('code' in o) && o.code.length > 0) {
			for(c in o.code)
				if('o' in o.code[c])
					renderClass(def, err, htm, o.code[c], mdep);
		} else {
			return err.push(['', obj, 'Stage is empty.']);
		}
	}

	function renderClass(def, err, htm, obj, mdep) {

		if(!(obj.o in def))
			return err.push(['', obj.o, 'Missing class definition!']);

		if(mdep < 0)
			return;

		for(var i = 0; i < obj.r; i++) {

			if(cdiv <= 0)
				return;

			var t = getTransform(obj.t, i + 1);
			var n = pickClassNum(def[obj.o]);
			htm.push('<span class="' + obj.o.replace('.', '') + n + '" ' +
								(t ? ' style="' + t + '"' : '') + '>');

			var o = def[obj.o][n];
			if('@maxdepth' in o)
				mdep = Math.min(mdep, parseInt(o['@maxdepth']));
			if(('code' in o) && o.code.length > 0) {
				for(c in o.code)
					if('o' in o.code[c])
						renderClass(def, err, htm, o.code[c], mdep - 1);
			} else {
				cdiv--;
			}

			htm.push('</span>');
		}
	}

	function previewLink(url) {
		$('#preview').html('');
		$('<iframe id="ipreview" frameborder="0" src="' + url +
			'" />').appendTo('#preview');
	}

	$('#render').click(function() {
		var def = {};
		var err = [];
		var css = [	'body,#stage {margin:0;padding:0;font-family:sans-serif;}'];
		var htm = [];

		parseSource($('#weavecss').val(), def, err, css);

		if(err.length === 0)
			renderStage(def, err, htm, '#stage', 100, 1000);

		if(err.length !== 0) {
			css = css.slice(0, 1);
			css.push('#stage {background:#444;width:100%;}');
			css.push('th,td {text-align:left;padding:1px;}');
			css.push('th {color:#fff;background:#444;font-weight:normal;}');
			css.push('td {background:#ddd}');

			htm = ['<tr><th>Line#</th><th>Source</th><th>Error</th></tr>'];
			for(e in err) {
				htm.push('<tr><td>' + err[e][0] + '</td>' +
										 '<td>' + err[e][1] + '</td>' +
										 '<td>' + err[e][2] + '</td></tr>');
			}
		} else {
			css.push('body,#stage {width:100%;height:100%;overflow:hidden;}');
			css.push('#stage{border-collapse:collapse;}');
			css.push('span {position:absolute;}');
			htm = ['<tr><td style="padding-left:50%;">' + htm.join('') + '</td></tr>'];
		}

		var	all = '<style>' + css.join('\n') + '</style>' +
							'<table id="stage">' + htm.join('') + '</table>';
		$('#preview').html('');
		$('<iframe id="ipreview" frameborder="0" />').load(function(){
			var bd = $('#ipreview').contents().find('body');
			bd.append(all);
			if($.browser.msie)
				bd.append('<p style="z-index:10;position:absolute;padding:10px;top:0;left:0;">Internet Explorer doesn\'t support CSS3 transforms yet.<br />Try the latest versions of Chrome, Firefox, Opera, or Safari.</p>');
			scalePreview();
		}).appendTo('#preview');
	});

	$('#presets').bind('change', function() {
		$('#weavecss').val($.trim($('#' + $(this).val()).html()));
		$('#render').click();
	}).change();

  $('#gallery').click(function() {
  	previewLink('https://www.reddit.com/r/weave/');
  });

  $('#help').click(function() {
  	previewLink('/weave/help');
  });

  $('#share,#publish').click(function() {
  	var t = this;
  	$.ajax({
  	  type: 'POST',
  		url:'/weave/save',
  		data:$.trim($('#weavecss').val()),
  		error:function() {
  			alert('Sorry! Cannot share this design right now.');
  		},
  		success:function(ret) {
  			if(ret === '')
  				return alert('Sorry! There was an error trying to save the design right now.');
  			var u = 'https://zetabee.com/weave/share/' + ret;
  			if(t.id == 'share')
					prompt('Copy-Paste the link below to share your design:', u);
				else {
			  	window.open('https://www.reddit.com/r/weave/submit?url=' + u);
				}
  		}
  	});
  });

};
